
/*
   Copyright (c) 1993,1994, UK, John M.Howells

   Program to display the version of Windows on which it is run.
   The program can be compiled for both 16-bit and 32-bit versions,
   but most of the code is unique to one version or the other, but
   it's easier to keep it all together.

   It will report:

	16-bit version:	 either Windows 3.1 'proper' or WOW (i.e. Windows on
			 Windows, the 16-bit emulation provided by Windows NT).
			 if running on WOW it also reports the version and
			 build of the underlying OS. if running on a system
			 with Win32s it also reports the version and build.

   or

	32-bit version:	 either Windows NT 'proper' or Chicago or Win32s
			 (the extension to allow 32-bit programs to run on
			 the 16-bit version of Windows 3.1x).

   The source of the information is as follows:

	Detect Windows-on-Windows:			Knowledgebase Article Q92395
	Detect Win32s:					Knowledgebase Article Q92395
	Detect the Version of Win32s:			Win32s Programmer's Reference
	Other Version information:			Knowledgebase Article Q92395
	Detect version reported to a DOS program:	DPMI specification
	Detect version of underlying Windows NT 3.1
		from 16-bit program:			NT 3.5 SDK CD \UCT\SAMPLES\INTEROP
	Detect version of underlying Windows NT 3.5
		from 16-bit program (and on
		Win32s v1.15 and later):		NT 3.5 SDK CD \UCT\SAMPLES\INTEROP
							
   When the program is running on a 16-bit version of Windows 3.1x or later
   it uses the DOS interface to check the version of Windows. It does this
   because Windows for Workgroups 3.11 tells lies (see MS article Q106271),
   and reports that it is Windows 3.10, but the DOS interface returns the
   correct version number. Even this does not work on Windows 3.11 (*not*
   Windows for Workgroups 3.11), as both interfaces, Windows and DOS,
   report 3.10, so the string from USER.EXE is also obtained!

   <PERSONAL OPINION ON>

   Development of this mixed-mode program is easier under the Borland environment,
   in my opinion, because the GUI can be used under 16-bit and 32-bit Windows to
   compile both 16-bit and 32-bit versions of the program, whereas the 32-bit GUI
   from Microsoft only works under WIndows NT, and you have to resort to the
   command-line tools to build a 32-bit version under 16-bit Windows, and you
   have to develop the program in effectively four modes, since the results are
   different from 16-bit and 32-bit of the program for the 16-bit and 32-bit OS's.
   
   <PERSONAL OPINION OFF>

   Modification History:

   1.0	94-03-31	JMH	Initial Version

   1.1	94-04-28	JMH	There are only two significant corrections. The first
				is for a silly mistake in omitting to initialize the
				bld_id string, though the original worked by accident!
				The other correction is for a better check for Version
				3.10 or later. The previous version checked for the
				major version greater than 3 and the minor version
   				greater than 10, which is not *quite* correct. The
   				other changes are cosmetic. Changed the check to set up
   				bld_id so no 'else' is needed. Corrected the spelling
   				of 'both'. Changed the	protected mode check in the
   				16-bit function to check for 3.10 version (3.11?) from
   				the use of a DPMI function to use of the GetWinFlags()
   				function. Changed the 32-bit check for NT to use a
   				macro, rather than an embedded constant. Defined a macro
   				for the cases where the DOS version check returns an
   				'unknown' result. Added 'argsused' pragma for WinMain()
				when compiling with Borland C++ 4.0. Corrected spelling
				of 'Windows'! Added the mode (Enhanced/Standard/Real)
				to the output in 16-bit mode.

   1.2	94-05-09	JMH	Modified to report the underlying OS version number
				when running on Windows NT. On the Windows NT 3.5
				(aka Daytona) Beta (build 612) the WOW GetVersion
				returns 3.10, presumably to kid 16-bit applications
				that they are running on 3.10, in case they might
				get confused if	they saw 3.50 returned. Here the
				underlying 32-bit version of GetVersion is called,
				as described on the Daytona Beta 612 CD in the
				\SCT\SAMPLES\INTEROP directory,	to get the true
				underlying Windows NT OS version and build.

   1.3	94-08-10	JMH	Modified to report the Build identity from USER.EXE
				on Windows 3.10 and later. The Windows 3.11 OS/2 kicker
				(*NOT* WFWG 3.11) still reports 3.10 to both Windows
				and DOS programs that ask what version, but the build
				string in USER.EXE actually says 3.11 correctly. Ain't
				it clever! The program cannot analyze the string and
				not print it out if it is the same as the version
				reported by GetVersion(), as it contains (incorrectly
				in my opinion) the string "3.1" and not "3.10". Also
				changed the string for the "True" version report to
				say "DOS reports ..." because who knows what may be
				going on in future versions! Are there any more places
				where a version	number is made available? Rearranged
				the output.

   1.4	94-10-27	JMH	Added the 32-bit GetVersionEx() interface to both
				16-bit and 32-bit versions. In each case the program
				does not assume that the function exists, so we have
				to use LoadLibrary(), and so on, to find the function
				and do run-time linking to it. The GetversionEx()
				function was not present in Windows NT 3.1 or in
				version 1.10 of Win32s, but is present in Windows
				NT 3.5, and in Version 1.15 of Win32s. Changed the
				16-bit version so that it detects whether or not the
				underlying OS supports the 32-bit thunk interface
				when trying to work out whether we are on a 32-bit
				OS for getting underlying version information, rather
				than relying on any flags. so that we *should* be
				independent of any particular OS, though probably
				it will only be valid on Windows NT. Changed the flow
				of the code slightly so that for each of the versions
				all output goes through a single call to MessageBox().
				Used conditional compilation to	allow the 'basic'
				version of the interfaces to be	tested on systems that
				support the Extended interface.	Changed so that the
				16-bit version always reports the DOS version, if
				available, when running on 16-bit host. Altered the
				Base Address of the Microsoft-built 32-bit version to
				400000 (the Borland tools already have this address
				as the default) to make the program a bit more Chicago
				compatible. Who would believe 'they' could invent so
				many ways of finding a simple thing like a version
				number! Must remember to get dates right this time!

   1.5	94-11-06	JMH	Slight change for compatibility with Visual C++ 2.0
   				(and presumably later versions of other tool suites),
   				to only define the OSVERSIONINFOA structure and its
				associated VER_PLATFORM_WIN32... #defines when the
				VER_PLATFORM_WIN32_NT macro is not already defined.
				Added version number to Dialog Box text. Added a
				header file for common version information.

   THIS PROGRAM HAS NOT BEEN RUN ON CHICAGO.

   The program does not yet detect whether it is running under Windows for
   Workgroups. The program can easily detect Windows for Workgroups on a
   networked system when the network is running, but this test is unreliable.
   The problem is that Windows for Workgroups does not require a network,
   and in such a mode there is no obvious difference from a standard Windows
   installation. The program could look for the presence of WFWNET.DRV in
   the WINDOWS\SYSTEM directory as an indication of Windows for Workgroups,
   but that is also an unreliable check, and we don't want inconsistent
   reports from a single system.

 */


#include	<windows.h>
#include	"win_ver.h"


/* #define	TEST_BASIC_IF 			/* remove comment to test basic 32-bit version */
						/* I/F's rather than the extended I/F's */


#define		PROG_NAME	"Windows Version Program : V" PROG_MAJOR_VERS_STRING "." PROG_MINOR_VERS_STRING



/* some of the things are defined here because they are not
   available in any of the header files that ship with the
   standard C/C++ development environments currently available,
   as presented here it will build with the retail versions of
   Microsoft VC++ 1.5 or 1.51 (from the VC++ 2 package) for
   16-bit and VC++ 1.10 (aka 1.0) 32-bit edition or VC++ 2.0,
   and the Borland C++ 4.02 package for both 16-bit and 32-bit. */



#define		WF_WINNT	0x4000		/* flag for WOW in 16-bit Windows */
#define		WV_WINNT	0x80000000	/* flag for NT in GetVersion() */

#define		W32SYS		"W32SYS.DLL"	/* name of Win32s DLL */
#define		WIN32SINF	"GETWIN32SINFO"	/* name of GetWin32sInfo function in W32SYS.DLL */

#define		KRNL386		"KRNL386.EXE"	/* name of Krnl386 EXE */
#define		LDLIB32W	"LOADLIBRARYEX32W"
						/* name of LoadLibraryEx32W function in KRNL386.EXE */
#define		CALLPROCEX	"_CALLPROCEX32W"/* name of CallProcEx32W function in Kernel32.DLL.
						   note that this does *not* exist on Windows NT 3.1,
						   only on Windows NT 3.5 and later. */

#define		USER_EXE	"USER.EXE"	/* name of USER.EXE file */
#define		BUILD_ID	516		/* number of Build string in USER.EXE */

#define		KERNEL32	"KERNEL32.DLL"	/* name of module that holds GetVersion(Ex) functions */
#define		GETVERN		"GetVersion"	/* name of GerVersion function in Kernel32.DLL */
#define		GETVERNEX	"GetVersionExA"	/* name of GetVersionEx function in Kernel32.DLL,
						   note that this is in W32sCOMB.DLL in Win32s,
						   but asking for it in the KERNEL32.DLL module
						   nevertheless still works properly */
						   
#define		VER_UNKNOWN	0xFFFF		/* value when can't get version from DOS */


#if	!defined( TEST_BASIC_IF )  		/* only relevent for new interface */

#if	!defined( VER_PLATFORM_WIN32_NT )	/* if already defined don't do again */

typedef	struct	_OSVERSIONINFOA	{		/* Extended Vern Info from GetVersionEx() */
	DWORD	dwOSVersionInfoSize;		/* set to size of structure */
	DWORD	dwMajorVersion;			/* OS major version */
	DWORD	dwMinorVersion;			/* OS minor version */
	DWORD	dwBuildNumber;			/* OS build number */
	DWORD	dwPlatformId;			/* OS platform type */
	char	szCSDVersion[ 128 ];		/* description string */
} OSVERSIONINFOA;

/* the values returned in the PlatformID field of the Extended Version
   information to indicate which version of the 32-bit platform is in use. */
#define		VER_PLATFORM_WIN32s		0	/* Win32s on Windows 3.1 */
#define		VER_PLATFORM_WIN32_WINDOWS	???	/* Win32 on Chicago (= 1 ???) */
#define		VER_PLATFORM_WIN32_NT		2	/* Windows NT */

#endif

#endif


#if	__BORLANDC__ >= 0x0452
/* on Borland C++ 4.x exclude exception handling code */
void	_ExceptInit( void )
{
	;
}
#endif


#ifdef	__BORLANDC__
#pragma	argsused
#endif

/* the function that drives the whole thing */
int PASCAL WinMain( HINSTANCE	hInstance,
		    HINSTANCE	hPrevInstance,
		    LPSTR	lpszCmdLine,
		    int		nCmdShow )
{
	DWORD	win_vers;
	char	buff[ 256 ];

#if	defined( WIN32 ) || defined( _WIN32 ) || defined( __WIN32__ )


	/* ====================================
	   program is being compiled for 32-bit
	   ==================================== */

#define	WIN_BITS	"32"	   
	   
	/* by comparison with the 16-bit code the 32-bit code is relatively
	   straightforward! There are only two functions GetVersion() and
	   GetVersionEx(). The latter is available on version 3.5 of Windows NT
	   but not on version 3.1 of Windows NT, and on version 1.15 build 103
	   of Win32s, but not on version 1.10 build 88 of Win32s. */


#if	!defined( TEST_BASIC_IF )

	HINSTANCE	hKernel32;	/* handle to 32-bit KERNEL32.DLL */
	FARPROC		Get_Version_Ex;	/* function ptr to GetVersionEx */
	OSVERSIONINFOA	os_vn_inf = { sizeof( OSVERSIONINFOA ) },
		       *os_vn_inf_ptr = &os_vn_inf;
					/* initialized Extended Version information,
					   and a pointer to it */

	/* first try for extended version information - the function is
	   run-time linked so that we can use simple information if
	   the extended information is not available on this version
	   of the OS. Here normal 32-bit functions are used. */
	if( ( hKernel32 = GetModuleHandle( KERNEL32 ) ) != NULL &&
	    ( Get_Version_Ex = GetProcAddress( hKernel32, GETVERNEX ) ) != NULL )
	{
		/* got Extended Version function ptr so use it */

		int	chrs;

		/* and get data */
		Get_Version_Ex( os_vn_inf_ptr );
		chrs = wsprintf( buff, "Version %d.%02d Build %d of %s\n"
				"%s",
				os_vn_inf.dwMajorVersion,
				os_vn_inf.dwMinorVersion,
				os_vn_inf.dwBuildNumber,
				os_vn_inf.dwPlatformId == VER_PLATFORM_WIN32_NT ? "Windows NT" :
					os_vn_inf.dwPlatformId == VER_PLATFORM_WIN32s ? "Win32s" :
						"Chicago",
				os_vn_inf.szCSDVersion
				);
		if( os_vn_inf.dwPlatformId == VER_PLATFORM_WIN32s )
		{
			/* if on Win32s append version of underlying 16-bit Windows */
			win_vers = GetVersion();
			wsprintf( buff + chrs,  "\n"
						"Running on\n"
						"Windows Version %d.%02d",
					LOBYTE( LOWORD( win_vers ) ),
					HIBYTE( LOWORD( win_vers ) ) );
		}
	}
	else
	{
	
		/* we could not get extended version information so go with basic */

#endif

		win_vers = GetVersion();
		if( ( win_vers & WV_WINNT ) == 0 )
		{
			/* this is real Windows NT */
			wsprintf( buff, "Version %d.%02d Build %d of Windows NT",
					LOBYTE( LOWORD( win_vers ) ),
					HIBYTE( LOWORD( win_vers ) ),
					HIWORD( win_vers ) & 0x7FFF );
		}
		else
		{
			if( LOBYTE( LOWORD( win_vers ) ) < 4 )
			{
				/* this is Win32s running on 16-bit Windows 3.1x,
				   because the program is running in 32-bit mode
				   and the extended information is not available it
				   cannot easily determine the version of Win32s as
				   that info comes from a 16-bit DLL - good innit.
				   the program *could* work out the version, but it
				   would require a 16-bit DLL to thunk to, and that
				   just is not worth the effort. */
				wsprintf( buff, "      Win32s running on Version %d.%02d\n"
						"\t      of Windows\n"
						"\n"
						"      Run 16-bit version for information\n"
						"                 on Version of Win32s\n"
						"and for additional 16-bit build information",
						LOBYTE( LOWORD( win_vers ) ),
						HIBYTE( LOWORD( win_vers ) ) );
			}
			else
			{
				/* this is Chicago - Windows 4.x - can we ever get here
				   as GetVersionEx should have already done its thing. */
				wsprintf( buff, "Chicago Version %d.%02d",
						LOBYTE( LOWORD( win_vers ) ),
						HIBYTE( LOWORD( win_vers ) ) );
			}
		}
		
#if	!defined( TEST_BASIC_IF )

	}
	
#endif

	/* ==========================
	   end of 32-bit Windows code
	   ========================== */


#else


	/* ============================
	   start of 16-bit Windows code
	   ============================ */

#define	WIN_BITS	"16"
	   
	/* compared with the 32-bit code this is horrible. First it tries
	   for a host 32-bit OS. If that fails it proceeds with 16-bit
	   detection, looking for Win32s as well as checking for version
	   numbers known to DOS and the USER.EXE module. */


	DWORD		win_flags;
	
	HINSTANCE	hKrnl386;		/* instance handle for Krnl386.EXE */

	win_vers = GetVersion();
	win_flags = GetWinFlags();
	/* detect whether the underlying OS can support the functions
	   that would allow a 16-bit program to thunk to an underlying
	   32-bit OS. we don't need the function address, so don't bother
	   to save it. this way we are not dependant on any particular
	   flags to tell us which version of the OS we are on. The
	   LoadLibraryEx32W function should exist on all such OS's,
	   though that probably means only a version of Windows NT. */
	if( /* ???? do we add this next test as well ??? - not for now on a '486 */
	    // win_flags & WF_WINNT ||
	    ( hKrnl386 = GetModuleHandle( KRNL386 ) ) != NULL &&
	      GetProcAddress( hKrnl386, LDLIB32W ) != NULL )
	{
		/* Program is running on Windows NT, or on some other system
		   (Chicago? - probably not) that supports LoadLibraryEx32W
		   so use it to get version data from the underlying 32-bit OS */

		DWORD		hKernel32;	/* handle to 32-bit DLL while thunking */
		DWORD		Get_Version;	/* ptr to 32-bit function that gets version. */

		/* prototypes for functions that permit the program to
		   thunk from this 16-bit world to a 32-bit module */
		DWORD	FAR	PASCAL	LoadLibraryEx32W( LPCSTR, DWORD, DWORD );
		BOOL	FAR	PASCAL	FreeLibrary32W( DWORD );
		DWORD	FAR	PASCAL	GetProcAddress32W( DWORD, LPCSTR );
		DWORD	FAR	PASCAL	CallProc32W( DWORD, DWORD, DWORD );		/* NT 3.1, etc. */

#if	!defined( TEST_BASIC_IF )

		DWORD	FAR	CDECL	CallProcEx32W( DWORD, DWORD, DWORD, ... );	/* NT 3.5, etc. */

		OSVERSIONINFOA	os_vn_inf = { sizeof( OSVERSIONINFOA ) },
			   far *os_vn_inf_ptr = &os_vn_inf;
						/* extended version information, initialized
						   appropriately, and a far ptr to it */
		DWORD		Get_Version_Ex;	/* ... and ptr to Extended Version function */

		/* first try for extended version information - we go through
		   this so that we can use simple information if the extended
		   information is not available on this version of the OS, as
		   Windows NT 3.1 does not provide Extended Version information.
		   
		   Here the program uses the thunk interface to call from the
		   16-bit version of the program 'through' to the 32-bit world.
		   the KRNL386 check is because the CallProcEx32W function did
		   not appear till Windows NT 3.5, so we can't use the extended
		   functions on Windows NT 3.1, and we don't want an unresolved
		   Dynalink message to stop the program, but we don't need to
		   save the address of the function. So, do checks and get data.
		   
		   We assume that if the CallProcEx32W() function does not exist
		   then nor does the GetVersionEx() function.
		   
		   The CallProcEx32W function passes 1 parameter that requires
		   address translation to the GetVersionEx() function.
		   
		   if any check fails, or we fail to get data, we go on to try
		   for basic data.
		   
		   Note, that it is *VITAL* that hKernel32 is obtained *FIRST*
		   so that is is *ALWAYS* done (the rest may not be!), as it
		   is assumed to be set up when getting the basic version. */
		if( ( hKernel32 = LoadLibraryEx32W( KERNEL32, NULL, 0 ) ) != NULL &&
		    ( Get_Version_Ex = GetProcAddress32W( hKernel32, GETVERNEX ) ) != NULL &&
		      GetProcAddress( hKrnl386, CALLPROCEX ) != NULL &&
		      CallProcEx32W( 1, 1, Get_Version_Ex, (DWORD)os_vn_inf_ptr ) == TRUE )
		{
			/* free library when done. */
			FreeLibrary32W( hKernel32 );
			/* prepare output buffer and give user information */
			wsprintf( buff, "WOW (Win16) reports OS Version %d.%2d\n"
					"running on\n"
					"Version %d.%02d Build %d of %s\n"
					"%s",
					LOBYTE( win_vers ), HIBYTE( win_vers ),
					(WORD)os_vn_inf.dwMajorVersion,
					(WORD)os_vn_inf.dwMinorVersion,
					(WORD)os_vn_inf.dwBuildNumber,
					os_vn_inf.dwPlatformId == VER_PLATFORM_WIN32_NT ? (LPCSTR)"Windows NT" :
						os_vn_inf.dwPlatformId == VER_PLATFORM_WIN32s ? (LPCSTR)"Win32s" :
							(LPCSTR)"Chicago" /* ???? */,
					(LPCSTR)os_vn_inf.szCSDVersion );
		}
		else
		{

#else

			/* if testing basic interfaces get library handle */
			hKernel32 = LoadLibraryEx32W( KERNEL32, NULL, 0 );

#endif

			/* failed to get extended information - try for basic */

			if( hKernel32 == NULL ||
			    ( Get_Version = GetProcAddress32W( hKernel32, GETVERN ) ) == NULL )
			{
				/* somehow we failed to get a handle to KERNEL32.DLL, or the
				   address of the GetVersion function - this should never happen!!! */
				wsprintf( buff, "Error\n"
						"\n"
						"No %s" KERNEL32 " module\n"
						"\n"
						"???? - How can this be - ????",
							hKernel32 != NULL ? GETVERN " function in" : "" );
				if( hKernel32 != NULL )
				{
					FreeLibrary32W( hKernel32 );
				}
			}
			else
			{
				DWORD	nt_vers;  	/* underlying OS version and build */

				/* get OS version number reported by the 32-bit OS.
				   no parameters, so no translation. note that here the
				   CallProc32W function is used, as the CallProcEx32W
				   function does not exist on Windows NT 3.1, and we
				   want this program to work on all Windows NT versions. */
				nt_vers = CallProc32W( Get_Version, 0, 0 );
				/*free the library when done */
				FreeLibrary32W( hKernel32 );
				/* prepare output buffer and give user information */
				wsprintf( buff, "WOW (Win16) reports OS Version %d.%2d\n"
						"running on\n"
						"Version %d.%02d Build %d of Windows NT\n",
							LOBYTE( LOWORD( win_vers ) ),
							HIBYTE( LOWORD( win_vers ) ),
							LOBYTE( LOWORD( nt_vers ) ),
							HIBYTE( LOWORD( nt_vers ) ),
							HIWORD( nt_vers ) & 0x7FFF );
			}
			
#if	!defined( TEST_BASIC_IF )

		}

#endif

	}
	else
	{
		/* Program is running on real 16-bit Windows 3.1x - here the program
		   can find out which version, if any, of Win32s is available, and
		   which particular build of Windows 3.1x this really is, but what
		   about Chicago ???? */

		/* data for Win32s */
		typedef	struct
		{
			BYTE	major;
			BYTE	minor;
			WORD	build;
			BOOL	debug;
		} WIN32SINFO, far *LPWIN32SINFO;

		/* data for Win32s */
		HANDLE		h_win32s;       	/* handle to Win32s library */
		FARPROC		getwin32s_info;		/* address of Win32s version function */
		WIN32SINFO	win32s_info;		/* the Win32s version information */
		char		temp[ 128 ];		/* filled with Win32s version - if any */

		/* data for USER.EXE build string */
		HANDLE		h_userexe;		/* handle to USER.EXE library */
		char		bld_temp[ 64 ] = "";	/* space for build string */
		char		bld_str[ 64 ] = "";	/* space for build result */

		/* data for DOS build string */
		char		bld_id[ 64 ] = "";	/* for true build information */
	
		WORD	get_dos_win_ver( void );	/* our function to ask DOS what
							   version of Windows it thinks */

		if( ( LOBYTE( LOWORD( win_vers ) ) << 8 ) + HIBYTE( LOWORD( win_vers ) ) >=
									0x030A )
		{
			/* reported version number is 3.10 or greater so
			   double check because Windows for Workgroups 3.11
			   and Windows 3.11 both report the version as 3.10 */
			WORD		dos_win_vers;

			dos_win_vers = get_dos_win_ver();
			if( dos_win_vers != VER_UNKNOWN )
			{
				/* setup DOS information */
				wsprintf( bld_id, "       DOS reports Windows %d.%d",
						HIBYTE( dos_win_vers ),
						LOBYTE( dos_win_vers ) );
			}
			/* don't disable No File Found error box - we should *always* find USER.EXE! */
			h_userexe = LoadLibrary( USER_EXE );
			if( h_userexe > HINSTANCE_ERROR )
			{
				int	temp;

				temp = LoadString( h_userexe, BUILD_ID, bld_temp, sizeof( bld_temp ) );
				if( temp != 0 )
				{
					wsprintf( bld_str, "  USER.EXE reports Windows %s", (LPSTR)bld_temp );
				}
				FreeLibrary( h_userexe );
			}
		}

		SetErrorMode( SEM_NOOPENFILEERRORBOX );
			/* disable "File Not Found" error box */

		/* now find out what (if any) version of Win32s is available */
		h_win32s = LoadLibrary( W32SYS );	/* try and load Win32s library */
		if( h_win32s > HINSTANCE_ERROR )
		{
			/* library loaded so get address of info function */
			getwin32s_info = GetProcAddress( h_win32s, WIN32SINF );
			if( getwin32s_info )
			{
				/* Win32s Version 1.1 or later installed */
				if( !(*getwin32s_info)((LPWIN32SINFO)&win32s_info))
				{
					wsprintf( temp, "\n Supporting Version %d.%d Build %d\n"
							"\t  of Win32s",
								win32s_info.major,
								win32s_info.minor,
								win32s_info.build );
				}
				else
				{
					wsprintf( temp, "???" );
				}
			}
			else
			{
				/* if no function then Win32s is version 1.0 */
				wsprintf( temp, "\nSupporting Version 1.0\n"
						"\tof Win32s" );
			}
			FreeLibrary( h_win32s );
		}
		else
		{
			/* no Win32s library */
			wsprintf( temp, "          Win32s not available" );
		}
		wsprintf( buff, "   Windows reports Version %d.%02d\n"
				"%s               in %s mode\n"
				"           on Version %d.%02d of DOS\n"
				"%s\n"
				"\n"
				"%s\n"
				"%s",
				LOBYTE( LOWORD( win_vers ) ),
				HIBYTE( LOWORD( win_vers ) ),
				(LPSTR)( ( win_flags & ( WF_ENHANCED | WF_STANDARD ) ) ?
						"" : "    " ),
				(LPSTR)( win_flags & WF_ENHANCED ? "Enhanced" :
					  win_flags & WF_STANDARD ? "Standard" : "Real" ),
				HIBYTE( HIWORD( win_vers ) ),
				LOBYTE( HIWORD( win_vers ) ),
				(LPSTR)temp,
				(LPSTR)bld_str,
				(LPSTR)bld_id );
	}
	
	/* ==========================
	   end of 16-bit Windows code
	   ========================== */


#endif

	MessageBox( NULL, buff, PROG_NAME " : " WIN_BITS "-bit",
				MB_OK | MB_ICONINFORMATION );
	return( FALSE );

}


#if	!defined( WIN32 ) && !defined( _WIN32 ) && !defined( __WIN32__ )


/* =========================================
   support function only used in 16-bit mode
   ========================================= */


#define		MPX_INT		0x2f		/* DOS Multiplex Interrupt */
#define		MPX_WIN_VER	0x160A		/* Get Windows Version */
#define		MPX_CHK_MODE	0x1686		/* Check for program mode */

#define		DPMI_INT	0x31		/* DPMI Interrupt */
#define		DPMI_REAL_INT	0x0300		/* DPMI call real-mode int */

/* this uses the DPMI interface to call real-mode INT 0x2F
   to get Windows version as seen by a DOS program. If the
   program directly uses INT 0x2F it gets a protected mode
   interupt that does not support the required interface.

   Thanks to Paul A LeBlanc of Borland's TeamB on CIS's BCPPWIN
   forum for suggesting the use of a DPMI real-mode interrupt
   for getting at the Windows version reported to a DOS program. */
WORD	get_dos_win_ver( void )
{
	/* DPMI Function 0x0300 (real-mode interrupt) register set */
	struct	real_regs
	{
		DWORD	reg_edi;
		DWORD	reg_esi;
		DWORD	reg_ebp;
		DWORD	reg_reserved;
		DWORD	reg_ebx;
		DWORD	reg_edx;
		DWORD	reg_ecx;
		DWORD	reg_eax;
		WORD	reg_status;
		WORD	reg_es;
		WORD	reg_ds;
		WORD	reg_fs;
		WORD	reg_gs;
		WORD	reg_ip;
		WORD	reg_cs;
		WORD	reg_sp;
		WORD	reg_ss;
	} real_regs;
	static	struct	real_regs far *real_reg_ptr;

	/* just make sure program is running in protected mode */
	if( ( GetWinFlags() & WF_PMODE ) == 0 )
	{
		/* real mode - this should never happen! */
		return( VER_UNKNOWN );
	}

	/* program is already running in protected mode,
	   so it is okay to make DPMI (INT 0x31) calls */
	real_reg_ptr = &real_regs;		/* init ptr to register set */

	real_regs.reg_ss = 0;			/* set SS:SP to zero ... */
	real_regs.reg_sp = 0;			/* ... just to make sure */
	real_regs.reg_eax = MPX_WIN_VER;	/* MPX fn to get WIN version */

	_asm	{
		les	di, real_reg_ptr	/* ptr to register set */
		mov	bx, MPX_INT		/* INT 0x2f - flags zero */
		mov	cx, 0			/* 0 words to real-mode stack */
		mov	ax, DPMI_REAL_INT	/* DPMI call real-mode int */
		int	DPMI_INT		/* go to it */
		jc	dpmi_bad		/* carry set on error */
		}

	return( LOWORD( real_regs.reg_ebx ) );	/* return word with WIN vers */

dpmi_bad:
	return( VER_UNKNOWN );		  	/* true version not known */
}

#endif
